/* Jeff DeMaagd
	July 20, 1998 */
/* This file holds all the advanced CAM functions */

#include <math.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "../geom/geom.h"
#include "camprot.h"



/************************************************************************/
short int start_program (char units, double part_clearance)
/* I have start_program and end_program to assure that general safety
precautions are taken.  I won't explain too much here, ask any NC
programmer. (7/25/1998) */
{
	fprintf (file_out, "%% (start program)\n");
	graphics (ON);
	z_pos (TOOLCHANGE);
	
	set_units (units);
	set_pos_mode (ABSOLUTE);
	scale_cancel ();
	
        set_data (START);
        parm_set (140, part_clearance);
        set_data (END);
        z_pos (CLEARANCE);
	fprintf (file_out, " (start machine)\n");
	return (0);
}



/************************************************************************/
void end_program (void)
{

	 /* according to some standards, not needed, but for safety anyway */
	if (cam_debug == YES)
	{
		fprintf (file_out, "(End program - safety stuff)\n");
	}

	coolant (OFF);
	spindle (OFF, 0.0);
	z_pos (TOOLCHANGE);

	/* well this IS for safety, I don't know if any machines remove
	these settings by default as I am doing here, but hey.  */

	comp (OFF);
	set_pos_mode (ABSOLUTE);
	scale_cancel ();

	machine_end_program ();
	return;
}



/************************************************************************/
char cut_line (struct line_seg ls1, double part_clearance, char z_down)
/* this is a surface line cut.  z_down has options of plunge or ramp. 
I'll go in diagonally at about 30 degrees because I don't want an end mill
to PLUNGE (7/31/1998) */
{
	char error = 0;
	double z_move;
	struct point p1;
	struct line l1;
	
	z_move = part_clearance - ls1.p1.z;

	z_pos (CLEARANCE);

	if (z_down == RAMP)	{
		l1.v1 = make_vector (ls1.p1, ls1.p2);
		l1.v1 =	unit_product (l1.v1);
		l1.v1 = vector_mul (l1.v1, z_move * 2);
		l1.p1 = ls1.p1;

		p1 = get_head (l1.p1, l1.v1);
		
		go_to_point (RAPID, XY, p1);
		go_to_point (LINEAR, XYZ, l1.p1);
	} else if (z_down == PLUNGE)	{
		go_to_point (RAPID,  XY, ls1.p1);
		go_to_point (LINEAR,  Z, ls1.p1);
//		go_to_point (LINEAR, XY, ls1.p2);
	} else if (z_down == NO)	{
		go_to_point (LINEAR, XYZ, ls1.p2);
	} else {
		fprintf (file_out, " (error: ACK! error in calling cut_line)");
		error = 1;
	}

	if (error == 0)	{
		go_to_point (LINEAR, XYZ, ls1.p2);
	}

	return (error);
}


/************************************************************************/
char cut_point (struct point p1, double part_clearance, char z_down)
/* this is a surface point cut/drill.  z_down has options of plunge or ramp. 
for ramp I go in diagonally at about 30 degrees because I don't want an
end mill (or other tool) to plunge (7/31/1998) */
{
	char error = 0;
	double z_move;
	struct point tp1, tp2;
	
	z_move = part_clearance - p1.z;
	tp1 = p1;	tp2 = p1;
	tp1.x = tp1.x - (2 * z_move);
	tp2.x = tp2.x + (2 * z_move);

	if (z_down != 0)
		z_pos (CLEARANCE);

	if (z_down == RAMP)	{
		go_to_point (RAPID, XY, tp1);
		go_to_point (LINEAR, XYZ, p1);
		go_to_point (LINEAR, XYZ, tp2);
	} else if (z_down == PLUNGE)	{
		go_to_point (RAPID, XY, p1);
		go_to_point (LINEAR, Z, p1);
	} else if (z_down == NO)	{
		go_to_point (LINEAR, XYZ, p1);
	} else {
		fprintf (file_out, " (error: ACK! error in calling cut_point)");
		error = 1;
	}

	return (error);
}



/************************************************************************/
char cut_arc (struct arc a1, double part_clearance, char z_down)
/* this is a surface arc cut.  z_down has options of plunge or ramp. 
I'll go in diagonally at about 30 degrees because I don't want an end mill
to plunge (9/1/1998) */
{
	char error = 0;
	double z_move;
	struct point p_start;
	
	z_move = part_clearance - a1.p1.z;
	p_start.x = a1.p1.x + (a1.r1 * cos ( radians (a1.a1)));
	p_start.y = a1.p1.y + (a1.r1 * sin ( radians (a1.a1)));
	p_start.z = a1.p1.z;

	z_pos (CLEARANCE);

	if (z_down == RAMP)	{
		go_to_point (RAPID, XY, p_start);
		go_to_point (LINEAR, Z, p_start);
		arc_motion (a1.p1, a1.r1, a1.a2, CCW);		
	} else if (z_down == PLUNGE)	{
		go_to_point (RAPID, XY, p_start);
		go_to_point (LINEAR, Z, p_start);
		arc_motion (a1.p1, a1.r1, a1.a2, CCW);		
	} else	{
		fprintf (file_out, " (error: Burp... I ate your cut_arc)");
		error = 1;
	}

	return (error);
}	/* cut_arc */



/************************************************************************/
void cut_3d_triangle (struct triangle t1,	double cut_fraction, 
			double tool_dia,	double part_clearance,
			double surface_z,	double cut_z_inc, 
			char cut_comp,		char z_down)
/* oh goodie, the intended "meat" of the project, why I even bothered in
the first place.  cut_fraction is the fraction of tool_dia that we want to
use between passes for pass width.  For cut_comp I have compensated and
uncompensated defined as an assinged number in camprot.h (7/25/1998) */
{
	int	max_loops, loop_count;
	struct	triangle tempt1 = t1;	//safety
	double	minz = t1.p1.z,		//the lowest z in the triangle 
		maxz = t1.p1.z,		//the highest z in the triangle
		int_z_inc,		// internal z increment, override request
		delta_z = z_down - t1.p1.z;
					//the difference needed to meet
					//the lowest point to the surface plane.

	z_pos (CLEARANCE);
	
	if (minz > t1.p2.z) {
		minz = t1.p2.z;
		delta_z = z_down - t1.p2.z;
	}
	
	if (minz > t1.p3.z) {
		minz = t1.p3.z;
		delta_z = z_down - t1.p3.z;
	}
	
	if (minz > surface_z) {
		printf("Error: tried to cut a triangle above object.\n");
		return;
	}
	
	max_loops = ceil (delta_z / cut_z_inc);	//# of loops to cut triangle
	int_z_inc = delta_z / max_loops;	//set internal 'z' increment
	
	
	if (maxz < t1.p2.z)	maxz = t1.p2.z;	//may not be needed.
	if (maxz < t1.p3.z)	maxz = t1.p3.z;

	tempt1.p1.z -= delta_z;		//could be -=
	tempt1.p2.z -= delta_z;
	tempt1.p3.z -= delta_z;
	
	for (loop_count = 1; loop_count <= max_loops; loop_count++) {
		tempt1.p1.z += int_z_inc;		//could be -=
		tempt1.p2.z += int_z_inc;
		tempt1.p3.z += int_z_inc;
		if ((tempt1.p1.z < surface_z) &&
		    (tempt1.p2.z < surface_z) &&
		    (tempt1.p3.z < surface_z))	{
		    cut_triangle (tempt1, cut_fraction, tool_dia, part_clearance, z_down);
		}
		    
	};

	return;
}



/************************************************************************/
void cut_triangle (struct triangle t1,	double cut_fraction, double tool_dia,
					double part_clearance, char z_down)

/* This function cuts an arbitrary 3D triangle in space.  This can be
dangerous unless you know what you are doing.  Try cut_3D_triangle first. 

For those trying to control the 'grain' of the cut, it lies parallel to
the direction of the line containing t1.p1 and t1.p2 

I realize I could save SOME machine time (up to 10-20% by doing a
triangular pattern vs this linear/parallel pattern, but it is a
matter of personal taste, plus, implementing that is relatively simple. */

{
	double cut_path_width = cut_fraction * part_clearance,
		tri_width;
	int	path_total_count,  //number of crossings needed in tool path
		path_count;	   //number of crossings taken
	
	if (cam_debug == YES)
	{
		fprintf (file_out, "(cut_triangle started)\n");
	}

	z_pos (CLEARANCE);

					// cut the 'borders' of the triange
	cut_line (make_line_seg(t1.p1, t1.p2), part_clearance, z_down);
	cut_line (make_line_seg(t1.p2, t1.p3), part_clearance, 0);
	cut_line (make_line_seg(t1.p3, t1.p1), part_clearance, 0);

				// the next set is needed for triangles
				// that are larger than the cutting path width.
				// I check width from all sides.

	tri_width = dist_from_line(make_line(t1.p1,t1.p2), t1.p3);
	if ((cut_path_width > tri_width)
	 && (cut_path_width > dist_from_line(make_line(t1.p2, t1.p3), t1.p1))
	 && (cut_path_width > dist_from_line(make_line(t1.p3, t1.p1), t1.p2)))
	{
			// make sure that my maximum path width isn't exceeded
		path_total_count = (int)ceil(tri_width / cut_path_width);

			// traverse each way for every other iteration
		for (path_count = 1; path_count < path_total_count; path_count++)
		{
			if ((path_count % 2) == 0) {
				go_to_point (LINEAR, XYZ, weighted_dist (t1.p1, t1.p3, path_count / path_total_count));
				go_to_point (LINEAR, XYZ, weighted_dist (t1.p2, t1.p3, path_count / path_total_count));
			} else {
				go_to_point (LINEAR, XYZ, weighted_dist (t1.p2, t1.p3, path_count / path_total_count));
				go_to_point (LINEAR, XYZ, weighted_dist (t1.p1, t1.p3, path_count / path_total_count));
			}
		}
	}

	return;
}



/************************************************************************/
void go_to_point (char pos_type, char coord_mask, struct point p1)
/* goes LINEARly to a point. A point used but the extraneous axes is ignored,
so the machine stays at its  present plane. (7/25/1998)
I added a coordinate mask so the program knows what points to ignore.  It is
similar to the unix octal chmod convention.  4 is X, 2 is Y, 1 is Z, so in
binary XY is 110, 6 in octal.  I ignore a zero coord_mask */
{
	char x_state = NO, y_state = NO, z_state = NO;

	if ((coord_mask & X) == X) { x_state = YES; }
	if ((coord_mask & Y) == Y) { y_state = YES; }
	if ((coord_mask & Z) == Z) { z_state = YES; }

	if (((coord_mask > 0) && (coord_mask < 8)) &&
		((pos_type == RAPID) || (pos_type == LINEAR)))
	{
		if (pos_type == RAPID)	{
			fprintf (file_out, "G00");
		} else	{	/* set to linear mode */
			fprintf (file_out, "G01");
		}

		if (x_state == YES)	{
			fprintf (file_out, " X%.4f", p1.x);
		}

		if (y_state == YES)	{
			fprintf (file_out, " Y%.4f", p1.y);
		}

		if (z_state == YES)	{
			fprintf (file_out, " Z%.4f", p1.z);
		}

		if (cam_debug == YES)	{
			fprintf (file_out, " (");

			if (pos_type == LINEAR)	{
				fprintf (file_out, "linear");
			} else	{
				fprintf (file_out, "rapid");
			}

			fprintf (file_out, " to point %3.3d %3.3d %3.3d)", x_state, y_state, z_state);
		}
	} else	{
		fprintf (file_out, " (error: I'm ignoring your go_to_point() !)");
	}
	fprintf (file_out, "\n");
} /* go_to_point */



/************************************************************************/
void arc_motion ( struct point center_pt, double radius, double end_angle, char direction)
{
	arc (direction);
	fprintf (file_out," XC%.4f YC%.4f R%.4f AB%.4f", 
			center_pt.x, center_pt.y, radius, end_angle);
	if (cam_debug == YES)	{
		fprintf ( file_out, " (cut an arc)");
	}
	fprintf ( file_out, "\n");

}



/************************************************************************/
/* end camadv.c */
/************************************************************************/

