/*	Jeff DeMaagd
	6/21/1998 

	This file holds the functions needed by the program camain.
	Make sure you have geomtype.h included. */

#include <stdio.h>
#include "../geom/geom.h"
#include "camprot.h"

/***********************************************************************/
/* this section outlines the G code functions */
/***********************************************************************/
void arc (char direction)
/* enters the code for a clockwise arc.  Make sure you declare an endpoint,
radius and have a feed rate declared */
{
	if (direction == CW)	{
		fprintf (file_out, "G02");
		if (cam_debug == YES)	{
			fprintf (file_out, "(clockwise arc)");
		}
	} else if (direction == CCW)	{
		fprintf (file_out, "G03");
		if (cam_debug == YES)	{
			fprintf (file_out, "(counter clockwise arc)");
		}
	} else 	{
		fprintf (file_out, "(error in setting arc type)\n");
	}
}

/***********************************************************************/
void dwell (double time)
/* enters the time to stop for "time" seconds (7/21/1998) */
{
	fprintf (file_out, "G04 F%f\n", time);
		/* an F, P or X may be used for setting dwell time */
}

/***********************************************************************/
void set_data (char action)
/* start and end make sense, but so does on and off (7/31/1998) */
{
	if (action == START)	{
		fprintf (file_out, "G10");
		if (cam_debug == YES)		{
			fprintf (file_out, " (Data set starting)");
		}
	} else if (action == END)	{
		fprintf (file_out, "G11");
		if (cam_debug == YES)	{
			fprintf (file_out, " (data set stopping)");
		}
	} else	{
		fprintf (file_out, "G11 (error: YOU want to do WHAT to MY data_set() function!?!)");
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void exact_stop (void)
/* machine goes to point doing an in position check.  My manual says that
this ensures sharp corners if needed on a part. (7/21/1998) */
{
	fprintf (file_out, "G09 ");
}

/***********************************************************************/
void clear_flz (void)
/* clears the floating zero setting (7/21/1998) */
{
	fprintf (file_out, "G12\n");
}

/***********************************************************************/
void set_units (char type)
/* sets dimensioning to inches, and feeds in inches per minute (ipm)
(7/21/1998) Generalized (7/31/1998) */

{
	if (type == INCH)	{
		fprintf (file_out, "G20");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (Set units to inches)");
		}
	} else if (type == METRIC)	{
		fprintf (file_out, "G21");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Set units to millimeters)");
		}
	} else	{
		fprintf (file_out, "G21 (error, invalid function entry)");
	}
	fprintf (file_out, "\n");
}

/***********************************************************************/
void safe_zone (char set)
/* sets or the safe zone, as defined in machine parameters. (7/21/1998) 
Generalized (7/31/1998) */
{
	if (set == ON)	{
		fprintf (file_out, "G22");
		if (cam_debug == YES)	{
			fprintf (file_out, " (safe zone on)");
		}
	} else if (set == OFF)	{
		fprintf (file_out, "G23");
		if (cam_debug == YES)	{
			fprintf (file_out, " (safe zone off)");
		}
	} else	{
		fprintf (file_out, "G22 (an error was made in safe_zone)");
	}

	fprintf (file_out, "\n");
}

/***********************************************************************/
void z_pos (char location)
/* parameter 140 sets clearance or "R" parameter in a canned cycle, for
clearance value, z coordinate is set by machine builder, machine just moves
tool vertically (7/21/1998) generalized (7/31/1998) */

{
	if (location == CLEARANCE)	{
/*		fprintf (file_out, "G31");	*/
		fprintf (file_out, "G00 Z0.10"); 
	/* hard coded clearance b/c !@#$# Milltronics @#$#@$ Mfg makes $#!+ */
		if (cam_debug == YES)	{
			fprintf (file_out, " (Z to clearance)");
		}
	} else if (location == TOOLCHANGE)	{
		fprintf (file_out, "G32");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Z to tool change)");
		}
	} else	{
		fprintf (file_out, "G32 (A grievous error has been made calling z_pos)");
			/* I assume that Z to toolchange is safer in general
			than either doing nothing or clearance setting */
	}
	fprintf (file_out, "\n");
}

/***********************************************************************/
void comp (char type)
/* Sets cutting tool compensation. (7/21/1998)
Generalized (7/31/1998) */
{
	if (type == OFF)	{
		fprintf (file_out, "G40");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Cancel Compensation)");
		}
	} else if (type == LEFT)	{
		fprintf (file_out, "G41");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Left Compensation)");
		}
	} else if (type == RIGHT)	{
		fprintf (file_out, "G42");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Right Compensation)");
		}
	}

	fprintf (file_out, "\n");
}

/***********************************************************************/
void scale_cancel (void)
/* see scale_set (7/21/1998) */
{
	fprintf (file_out, "G50");
        if (cam_debug == YES)	{
                fprintf (file_out, " (Cancel Scale)");
        }
	fprintf (file_out, "\n");
}

/***********************************************************************/
void scale_set (struct point scale_cen, vector_t scale_mag)
/* see turns on the scaling of a program.  The center of the scale is at
scale_cen, the scaling magnitudes of each axis is in each component of
scale_mag (7/21/1998) */
{
	fprintf (file_out, "G51 I%f J%f K%f X%f Y%f Z%f\n",
			scale_cen.x, scale_cen.y, scale_cen.z,
			scale_mag.i, scale_mag.j, scale_mag.k);
}

/***********************************************************************/
void set_pos_mode (char type)
/* sets positioning mode (7/31/1998)*/
{
	if (type == ABSOLUTE)	{
		fprintf (file_out, "G90");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Set absolute mode)");
		}
	} else if (type == RELATIVE)	{
		fprintf (file_out, "G91");
		if (cam_debug == YES)	{
			fprintf (file_out, " (Set incremental mode)");
		}
	} else	{
		fprintf (file_out, "G90 (error, set_pos() positioning type unclear)");
	}
	fprintf (file_out, "\n");
} /* set_pos */

