/*	Jeff DeMaagd
	6/21/1998 

	This file holds the functions needed by the program camain.
	Make sure you have geomtype.h included. 

	Multiple M commands per line are possible, but some controls do 
	not have the order defined.  I prefer one command per line, thanks.
	(7/2/1998) */

#include <stdio.h>
#include "../geom/geom.h"
#include "camprot.h"

/***********************************************************************/
/* "M" code functions */
/***********************************************************************/
void stop (char type)
/* M00 machine function.  This forces the operator to push cycle start
in order for the program to continue.  This gives a pause so the part
can be rearranged or such. */
{
	if (type == PROGRAM)
	{
		fprintf (file_out, "M00");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (program stop)");
		}
	}
	else if (type == OPTIONAL)
	{
		fprintf (file_out, "M01");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (optional program stop)");
		}
	}	
	else
	{
		fprintf (file_out, "M00 (somebody made a boo-boo in calling stop()! )");
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void end_program_machine_on (void)
/* M02 machine function.  Like M30, M99, leaves spindle and coolant on,
M30 turns both off.  Use this if you want to reduce cycle time, but 
please only as using a program as a subroutine. (7/2/1998) */
{
	fprintf (file_out, "M02\n");
	return;
}

/***********************************************************************/
void spindle (char direction, double speed)
/* Sets the spindle motion.  Use the "Sxxxx" command to set up the spindle
RPM before this command is executed.  (7/2/1998) generalized (7/30/1998) */
{
	if (direction == CW)
	{
		fprintf (file_out, "M03 S%.2f", speed);
		if (cam_debug == YES)
		{
			fprintf (file_out, " (spindle clockwise)");
		}
	}
	else if (direction == CCW)
	{
		fprintf (file_out, "M04 S%.2f", speed);
		if (cam_debug == YES)
		{
			fprintf (file_out, " (spindle counter-clockwise)");
		}
	}
	else if (direction == OFF)
	{
		fprintf (file_out, "M05 S%.2f", speed);
		if (cam_debug == YES)
		{
			fprintf (file_out, " (spindle off)");
		}
	}
	else if (direction == ORIENT)
		/* Rotates the spindle of machine (not rotary table), so that
		it is aligned to accept a tool change.  This also works on
		other machines with an orient function (7/2/1998) */
	{
		fprintf (file_out, "M19");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (orient spindle)");
		}
	}
	else
	{
		fprintf (file_out, "M03 S1000.0 (error: gee, i'm confused by the spindle() function entry)");
	}

	fprintf (file_out, "\n");
	return;
}	/* spindle */

/***********************************************************************/
void tool_change (int tool)
/* M06 machine function.  Tool change.  If there is a physical tool change
slot available at (int) tool, the machine changes tools.  Some machines also
have a manual tool change flag to force a manual tool change.  Please only
change tools when M06 is called.  Also increasingly more machines have an
automatic tool changer, I wouldn't suggest buying one with out at least 6
tool positions, and definitely not without one period unless the use is VERY
specific.  (7/2/1998) */
{
	fprintf (file_out, "M06 T%d", tool);
	if (cam_debug == YES)
	{
		fprintf (file_out, " (Change to tool %d)", tool);
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void coolant (char setting)
/* M07 machine function.  Turns mist cooling on (7/2/1998) */
{
	if (setting == MIST)
	{
		fprintf (file_out, "M07");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (coolant set to mist)");
		}
	}
	else if (setting == FLOOD)
	{
		fprintf (file_out, "M08");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (coolant set to flood)");
		}
	}
	else if (setting == OFF)
	{
		fprintf (file_out, "M09");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (coolant off)");
		}
	}
	else
	{
		fprintf (file_out, "M08 (error: argghh! in calling coolant()! )");
	}	

	fprintf (file_out, "\n");
	
return;
}

/***********************************************************************/
void rotary_table (char setting)
/* M10/M11 machine functions.  (En|de-en)ergizes the clamp or table for a
rotary table, if it exists. (7/2/1998) */
{
	if (setting == CLAMP)
	{
		fprintf (file_out, "M10");
	}
	else if (setting == UNCLAMP)
	{
		fprintf (file_out, "M11");
	}
	else
	{
		fprintf (file_out, "M11");
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void machine_end_program (void)
/* M30 machine function.  Like M30, M99, leaves spindle and coolant on,
M30 turns both off.  Use this if you want to reduce cycle time, but 
please only as using a program as a subroutine. (7/2/1998) */
{
	fprintf (file_out, "M30\n%%");
	if (cam_debug == YES)
	{
		fprintf (file_out, " (End Program)");
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void test_wait_channel (char channel)
/* M32 machine function.  This is for add-on functions to the machine
I have access to.  It basically sends an output and wait until a signal is
returned. */
{
	if (channel == M32)
	{
		fprintf (file_out, "M32\n");
	}
	else if (channel == M92)
	{
		fprintf (file_out, "M92\n");
	}
	else
	{
		fprintf (file_out, " (error: Burrrp!  I ate you test_wait_channel() command)");
	}
	return;
}

/***********************************************************************/
void graphics (char mode)
/* M90 machine function.  Turns the machine graphics plotting off, helps
reduce excessive lines. A good idea if redundant passes are taken and
graphics memory may be exceeded in the course of program run. 
(7/2/1998) */
{
	if (mode == OFF)
	{
		fprintf (file_out, "M90");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (graphics off)");
		}
	}
	else if (mode == ON)
	{
		fprintf (file_out, "M91");
		if (cam_debug == YES)
		{
			fprintf (file_out, " (graphics on)");
		}
	}
	else
	{
		fprintf (file_out, "M90 (error: graphics() command not understood)");
	}
	fprintf (file_out, "\n");
	return;
}

/***********************************************************************/
void sweep_3D_off (void)
/* M93 machine function.  Turns off the 3d sweep function.  It requires
specific built-in parameter numbers, so I won't really try to use this,
unless I know it is relatively standard. (7/2/1998) */
{
	fprintf (file_out, "M93\n");
	return;
}

/***********************************************************************/
void sweep_3D_on (void)
/* M94 machine function.  Turns on the 3d sweep function.  It requires
specific built-in parameter numbers, so I won't really try to use this,
unless I know it is relatively standard. (7/2/1998) */
{
	fprintf (file_out, "M94\n");
	return;
}

/***********************************************************************/
void tapered_wall (void)
/* M95 machine function.   It requires specific built-in parameter numbers,
so I won't really try to use this, unless I know it is relatively standard.
(7/2/1998) */
{
	fprintf (file_out, "M95\n");
	return;
}

/***********************************************************************/
void rounded_wall (void)
/* M96 machine function.   It requires specific built-in parameter numbers,
so I won't really try to use this, unless I know it is relatively standard.
(7/2/1998) */
{
	fprintf (file_out, "M96\n");
	return;
}

/***********************************************************************/
void pocket_clear (void)
/* M97 machine function.   It requires specific built-in parameter numbers,
so I won't really try to use this, unless I know it is relatively standard.
(7/2/1998) */
{
	fprintf (file_out, "M97\n");
	return;
}

/***********************************************************************/
void call_subprogram (short int number, short int loops)
/* M98 machine function.  In my machine's manual, "Nesting with up to 50
loops is allowed."  Wether that means 50 loops total if you sub-nest or
50 level subnesting, I don't know yet.  (7/2/1998) */
{
	fprintf (file_out, "M98 P%d L%d", number, loops);
	if (cam_debug == YES)
	{
		fprintf (file_out, " (calling sub program)");
	}
	
	return;
}

/***********************************************************************/
void end_subprogram (void)
/* M99 machine function.  Like M30, M99, but leaves spindle and coolant on,
M30 turns both off.  Use this if you want to reduce cycle time, but 
please only as using a program as a subroutine. (7/2/1998) */
{
	fprintf (file_out, "M99\n");
	return;
}
/***********************************************************************/
/* end of cammcode.h */
/***********************************************************************/

